import {
  callUtilMethod,
  getArrayQueryKey,
  getContextClient,
  setContextClient,
  splitUserOptions
} from "./chunk-APIE3FUV.mjs";

// src/index.ts
export * from "@trpc/client";

// src/createTRPCSvelte.ts
import {
  createInfiniteQuery,
  createMutation,
  createQuery
} from "@tanstack/svelte-query";
import {
  createTRPCUntypedClient
} from "@trpc/client";
import {
  createFlatProxy,
  createRecursiveProxy
} from "@trpc/server/shared";
import { BROWSER } from "esm-env";
import { writable } from "svelte/store";
var clientMethods = {
  query: [1, "query"],
  mutation: [0, "any"],
  infiniteQuery: [1, "infinite"],
  mutate: [0, "any"],
  fetchQuery: [1, "query"],
  prefetchQuery: [1, "query"],
  fetchInfiniteQuery: [1, "infinite"],
  prefetchInfiniteQuery: [1, "infinite"],
  subscription: [1, "query"]
};
function createHooksInternalProxy(client, opts) {
  const queryClient = opts.queryClient;
  return createFlatProxy((firstPath) => {
    switch (firstPath) {
      case "queryClient": {
        return queryClient;
      }
    }
    return createRecursiveProxy(({ path, args: unknownArgs }) => {
      path.unshift(firstPath);
      const method = path.pop();
      const joinedPath = path.join(".");
      const methodData = clientMethods[method];
      if (!methodData) {
        const utils = path.pop();
        if (utils === "utils") {
          return callUtilMethod(
            client,
            queryClient,
            path,
            method,
            unknownArgs
          );
        }
        throw new TypeError(`trpc.${joinedPath}.${method} is not a function`);
      }
      const args = unknownArgs;
      const [optionIndex, queryType] = methodData;
      const options = args[optionIndex];
      const [trpcOptions, tanstackQueryOptions] = splitUserOptions(options);
      const key = getArrayQueryKey(
        path,
        method === "mutation" ? void 0 : args[0],
        queryType
      );
      const enabled = tanstackQueryOptions?.enabled !== false;
      switch (method) {
        case "query":
          return createQuery({
            ...tanstackQueryOptions,
            enabled,
            queryKey: key,
            queryFn: () => client.query(joinedPath, args[0], trpcOptions)
          });
        case "mutation": {
          return createMutation({
            ...tanstackQueryOptions,
            mutationKey: key,
            mutationFn: (variables) => client.mutation(joinedPath, variables, trpcOptions)
          });
        }
        case "subscription": {
          const state = writable("idle");
          const data = writable(void 0);
          const error = writable(
            void 0
          );
          const enabled2 = (options?.enabled ?? true) && BROWSER;
          if (!enabled2) {
            return {
              state,
              data,
              error,
              unsubscribe: () => void 0
            };
          }
          const subscription = client.subscription(joinedPath, args[0], {
            onStarted: () => {
              state.set("running");
            },
            onData: (data_) => {
              data.set(data_);
            },
            onError: (error_) => {
              error.set(error_);
            }
          });
          const unsubscribe = () => {
            state.set("stopped");
            subscription.unsubscribe();
          };
          return {
            state,
            data,
            error,
            unsubscribe
          };
        }
        case "infiniteQuery":
          return createInfiniteQuery({
            ...tanstackQueryOptions,
            enabled,
            queryKey: key,
            queryFn: (context) => {
              const input = { ...args[0], cursor: context.pageParam };
              return client.query(joinedPath, input, trpcOptions);
            }
          });
        case "mutate": {
          return client.mutation(joinedPath, args[0], trpcOptions);
        }
        case "fetchQuery": {
          return queryClient.fetchQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: () => client.query(joinedPath, args[0], trpcOptions)
          });
        }
        case "prefetchQuery": {
          return queryClient.prefetchQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: () => client.query(joinedPath, args[0], trpcOptions)
          });
        }
        case "fetchInfiniteQuery": {
          return queryClient.fetchInfiniteQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: (context) => {
              const input = { ...args[0], cursor: context.pageParam };
              return client.query(joinedPath, input, trpcOptions);
            }
          });
        }
        case "prefetchInfiniteQuery": {
          return queryClient.prefetchInfiniteQuery({
            ...tanstackQueryOptions,
            queryKey: key,
            queryFn: (context) => {
              const input = { ...args[0], cursor: context.pageParam };
              return client.query(joinedPath, input, trpcOptions);
            }
          });
        }
        default:
          throw new TypeError(`trpc.${joinedPath}.${method} is not a function`);
      }
    });
  });
}
function createTRPCSvelteClient(opts) {
  const client = createTRPCUntypedClient(opts);
  const proxy = createHooksInternalProxy(client, opts);
  return proxy;
}
function createTRPCSvelte() {
  const createClient = createTRPCSvelteClient;
  const setContextClient2 = setContextClient;
  const getContextClient2 = getContextClient;
  return {
    createClient,
    setContextClient: setContextClient2,
    getContextClient: getContextClient2
  };
}
export {
  createTRPCSvelte,
  createTRPCSvelteClient
};
